from contextlib import contextmanager
import os
from StringIO import StringIO
import unittest
from urllib2 import URLError

import mock
import pymongo
from pyramid import testing
from webtest import TestApp

from charmworld import main
from charmworld.models import getconnection
from charmworld.models import getdb
from charmworld.testing.factory import makeSSOResponse
from charmworld.utils import get_ini


INI = get_ini()
MONGO_URL = os.environ.get('TEST_MONGODB')
MONGO_HOST = MONGO_URL.split(':')[0]
MONGO_PORT = int(MONGO_URL.split(':')[1])
MONGO_DATABASE = INI.get('mongo.database')
# Update the ini values with the database fired up by the nosemongo plugin.
INI['mongo.host'] = MONGO_HOST
INI['mongo.port'] = MONGO_PORT


def _drop_databases():
    connection = pymongo.Connection(
        MONGO_HOST, MONGO_PORT)
    connection.drop_database(MONGO_DATABASE)


class MongoTestBase(unittest.TestCase):

    def setUp(self):
        self.connection = getconnection({
            'mongo.host': MONGO_HOST,
            'mongo.port': MONGO_PORT,
        }, safe=True)
        self.db = getdb(self.connection, MONGO_DATABASE)

    def tearDown(self):
        # Drop the Mongo database.
        _drop_databases()


class WebTestBase(MongoTestBase):

    def setUp(self):
        super(WebTestBase, self).setUp()
        app = main({}, **INI)
        self.app = TestApp(app)

    def tearDown(self):
        self.app = None
        _drop_databases()
        super(WebTestBase, self).tearDown()


class ViewTestBase(MongoTestBase):

    def setUp(self):
        super(ViewTestBase, self).setUp()
        self.config = testing.setUp()

    def getRequest(self, params=None, post=None):
        request = testing.DummyRequest(params=params, post=post)
        request.db = self.db
        return request

    def tearDown(self):
        testing.tearDown()
        super(ViewTestBase, self).tearDown()


class RemoteURLBase(MongoTestBase):

    url_prefix = "https://jenkins.qa.ubuntu.com/job/"
    url_debug = False

    def setUp(self):
        super(RemoteURLBase, self).setUp()
        self.url_result_map = {"": ""}

    def lookup_url_result(self, url):
        if self.url_debug:
            print "url", url
        short_url = url[len(self.url_prefix):]
        if short_url in self.url_result_map:
            return StringIO(self.url_result_map[short_url])
        raise URLError(url)

    def invoke(self, func, *args, **kw):
        with mock.patch("urllib2.urlopen") as urlopen:
            urlopen.side_effect = self.lookup_url_result
            return func(*args, **kw)


@contextmanager
def login_request(app, groups=None):
    """Context manager to help functionally test a logged in user account.

    :param app: The application constructed for testing. Usually self.app when
                using WebTestBase.
    :return response: The post-logged in WebTest response object.

    """
    sso_resp = makeSSOResponse(groups=groups)
    resp = app.get(
        '/auth_callback',
        params=sso_resp,
        status=302)
    resp = resp.follow()
    yield resp
