import json
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound
import time
from webob import Response

from charmworld.jobs.config import CHARM_INDEX_DIR
from charmworld.views import SEARCH_FIELDS
from charmworld.views.helpers import result_sorter


def do_search(terms, allow_all_fields=False):
    import xappy
    searcher = xappy.SearchConnection(CHARM_INDEX_DIR)
    if allow_all_fields:
        deny = []
    else:
        deny = ['relations', 'config', 'changes']
    query = searcher.query_parse(
        terms,
        allow=SEARCH_FIELDS,
        allow_wildcards=True,
        default_deny=deny)

    t = time.time()
    results = query.search(0, 30)
    search_time = time.time() - t
    search_time = u"%.5f" % search_time
    return {"results": results,
            "search_time": search_time,
            "charm_total": searcher.get_doccount()}


@view_config(
    route_name="search",
    renderer="charmworld:templates/search.pt")
def search(request):
    if not "search_text" in request.params:
        url = request.route_url("home")
        return HTTPFound(location=url)
    # XXX bug=1099856 jcsackett Jan 1 2012: `allow_all_fields` must be true
    # for the template, else the rendering engine blows up on
    # `results.matches_human_readable_estimate`
    return do_search(request.params["search_text"], allow_all_fields=True)


@view_config(
    route_name="search-json")
def search_json(request):
    data = do_search(request.params["search_text"])
    results = data['results']
    charms = []
    for r in results:
        if not 'store_url' in r.data:
            continue

        charms.append({
            'store_url': r.data['store_url'][0],
            'name': r.data['name'][0],
            'summary': r.data['summary'][0],
            'series': r.data['series'][0],
            'owner': r.data['owner'][0],
            'data_url': '%s/json' % (
                # request.host_url,
                r.data['short_url'][0]),
            'relevance': r.weight,
        })

        if 'subordinate' in r.data:
            charms[-1]['subordinate'] = True

    charms.sort(result_sorter)

    data.update({"results": charms,
                 "matches": results.matches_estimated,
                 "results_size": len(charms)})

    return Response(
        json.dumps(data, indent=2),
        headerlist=[
            ("Access-Control-Allow-Origin", "*"),
            ("Access-Control-Allow-Headers", "X-Requested-With")],
        content_type=u"application/json")
