from mock import patch
from charmworld.testing import ViewTestBase
from charmworld.views import search


class ResultFake(list):
    def __init__(self, results):
        super(ResultFake, self).__init__(results)
        self.results = results
        self.matches_estimated = len(results)


RETURN_VAL = {
    'results': ResultFake([]),
    'search_time': 0.0,
    'charm_total': 0,
}


class SearchTests(ViewTestBase):

    @patch('charmworld.views.search.do_search',
           autospec=True, return_value=RETURN_VAL)
    def test_search(self, mock):
        terms = {"search_text": 'foo'}
        keys = ['charm_total', 'results', 'search_time']
        request = self.getRequest(params=terms)
        response = search.search(request)

        mock.assert_called_with('foo', allow_all_fields=True)
        self.assertEqual(keys, sorted(response.keys()))
        self.assertEqual(0, response['charm_total'])
        self.assertEqual([], response['results'])
        self.assertEqual(0.0, response['search_time'])

    @patch('charmworld.views.search.do_search',
           autospec=True, return_value=RETURN_VAL)
    def test_search_json(self, mock):
        # The search json view substitutes the search result object with the
        # charms and adds some metadata.
        terms = {"search_text": 'foo'}
        request = self.getRequest(params=terms)
        keys = ['charm_total', 'matches', 'results', 'results_size',
                'search_time']
        json = search.search_json(request).json

        mock.assert_called_with('foo')
        self.assertEqual(keys, sorted(json.keys()))
        self.assertEqual(0, json['charm_total'])
        self.assertEqual(0, json['matches'])
        self.assertEqual([], json['results'])
        self.assertEqual(0, json['results_size'])
        self.assertEqual(0.0, json['search_time'])
