import pymongo
from pyramid.view import view_config

from charmworld import utils
from charmworld.views.helpers import format_proof
from charmworld.views.helpers import sub_filter


# 3m
REPORT_CACHE = 180


@view_config(
    route_name="tools-core-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def core_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.core_review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def charm_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-store-missing",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/store-missing.pt")
def charm_store_missing(request):
    queue = list(
        request.db.charms.find({
            "store_data.errors": {"$exists": True}})
    )

    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-maintainer-missing",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/charm-collection.pt")
def charm_maintainer_missing(request):
    charms = list(
        request.db.charms.find(
            sub_filter({"maintainer": {"$exists": False},
                         "owner": "charmers"},
                        request),
            sort=[("series", pymongo.DESCENDING), ("name", pymongo.ASCENDING)]
        ))

    return {'charms': charms, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-proof-errors",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/charm-proof-errors.pt")
def proof_errors(request):
    query, sub_only = sub_filter(
        {"proof.e": {"$exists": True}, "owner": "charmers"}, request)

    charms = list(
        request.db.charms.find(
            query,
            sort=[("series", pymongo.DESCENDING), ("name", pymongo.ASCENDING)]
        ))

    return {
        'charms': charms,
        'format_proof': format_proof,
        'sub_only': sub_only,
    }
